/*	Copyright (C) 2003-2013 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "oplab/oplab.pmh"

#define FE_IMO_DEBUG		FALSE

using namespace fe;

void ImportOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	catalog<String>("format")="auto";
	catalog<String>("format","label")="Format";
	catalog<String>("format","choice:0")="auto";
	catalog<String>("format","label:0")="Auto";
	catalog<String>("format","choice:1")="xml";
	catalog<String>("format","label:1")="XML";

	catalog<String>("loadfile")="";
	catalog<String>("loadfile","label")="Load File";

	catalog<String>("filterFile")="";
	catalog<String>("filterFile","label")="Filter File";
	catalog<bool>("filterFile","joined")=true;

	catalog<String>("filterSpec")="";
	catalog<String>("filterSpec","label")="Spec";

	catalog<Real>("frame")=0.0;
	catalog<String>("frame","label")="Frame";

	catalog< sp<Component> >("Output Surface");
	catalog<String>("Output Surface","IO")="output";

	m_spScope=registry()->create("Scope");
}

void ImportOp::handle(Record& a_rSignal)
{
	const String loadfile=catalog<String>("loadfile");
	const String filterFile=catalog<String>("filterFile");
	const String filterSpec=catalog<String>("filterSpec");

	if(loadfile.empty())
	{
		return;
	}

	String format=catalog<String>("format");
	if(format=="auto")
	{
		format=String(strrchr(loadfile.c_str(),'.')).prechop(".");
	}

	sp<StringFilterI> spStringFilter;
	if(!filterFile.empty())
	{
		const String filterFormat=
			String(strrchr(filterFile.c_str(),'.')).prechop(".");

		spStringFilter=registry()->create("StringFilterI.*.*."+filterFormat);
		if(spStringFilter.isValid())
		{
			spStringFilter->configure(filterSpec);
			spStringFilter->load(filterFile);
		}
	}

	Real frame=catalog<Real>("frame");
	if(frame<startFrame(a_rSignal) || frame>endFrame(a_rSignal))
	{
		frame=currentFrame(a_rSignal);
	}

	const I32 intFrame=frame;
	const String text=print(intFrame);
	const String modfile=loadfile.substitute("$F",text);

#if FE_IMO_DEBUG
	feLog("ImportOp::handle %.6G\n",frame);
#endif

	sp<SurfaceAccessibleI> spOutputAccessible;
	if(!accessOutput(spOutputAccessible,a_rSignal)) return;

	sp<SurfaceAccessibleI> spLoadAccessible;

	spLoadAccessible=registry()->create("SurfaceAccessibleI.*.*."+format);
	if(spLoadAccessible.isNull())
	{
		spLoadAccessible=registry()->create("SurfaceAccessibleI.*.*.xml");
	}
	if(spLoadAccessible.isNull())
	{
		spLoadAccessible=registry()->create("SurfaceAccessibleI");
	}

	spLoadAccessible->bind(m_spScope);
	if(!spLoadAccessible->load(modfile,spStringFilter))
	{
		feLog("ImportOp::handle failed to load \"%s\" filter \"%s\"\n",
				modfile.c_str(),filterFile.c_str());
		spLoadAccessible=NULL;
	}

	if(spLoadAccessible.isValid())
	{

		sp<SurfaceAccessibleI> spOutputAccessible;
		if(!accessOutput(spOutputAccessible,a_rSignal)) return;

		spOutputAccessible->copy(spLoadAccessible);
	}
}
